'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.hexColorToArray = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.createImageBuffer = createImageBuffer;
exports.drawPixel = drawPixel;
exports.drawRect = drawRect;
exports.drawBuffer = drawBuffer;
exports.mapPixels = mapPixels;
exports.replaceColor = replaceColor;
exports.getColor = getColor;

var _utils = require('./utils.js');

var _constants = require('./constants.js');

/**
 * Creates a new empty canvas.
 */
function createImageBuffer(width, height) {
	var hasAlpha = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

	var channels = (0, _utils.getChannelCount)(hasAlpha);
	return new Uint8Array(width * height * channels).fill(0x00);
}

/**
 * Sets one pixels color on the canvas.
 */
function drawPixel(buffer, width, hasAlpha, pixelX, pixelY, pixelColor) {
	return mapPixels(buffer, width, hasAlpha, function (x, y, bytePos, color) {
		return x === pixelX && y === pixelY ? pixelColor : color;
	});
}

/**
 * Draws a rectangle.
 */
function drawRect(buffer, imgWidth, hasAlpha, offsetX, offsetY, width, height, rectColor) {
	return mapPixels(buffer, imgWidth, hasAlpha, function (x, y, bytePos, color) {
		return x >= offsetX && x < width + offsetX && y >= offsetY && y < height + offsetY ? rectColor : color;
	});
}

/**
 * Draws a canvas on another canvas.
 */
function drawBuffer(destination, destWidth, destHasAlpha, source, srcWidth, srcHasAlpha, offsetX, offsetY) {
	return mapPixels(destination, destWidth, destHasAlpha, function (x, y, bytePos, color) {
		var srcHeight = (0, _utils.getHeight)(source, srcWidth, srcHasAlpha);
		if (x >= offsetX && x < srcWidth + offsetX && y >= offsetY && y < srcHeight + offsetY) {
			return (0, _utils.blendColor)(getColor(destination, destWidth, destHasAlpha, x, y), getColor(source, srcWidth, srcHasAlpha, x - offsetX, y - offsetY));
		} else {
			return color;
		}
	});
}

function mapPixels(buffer, width, hasAlpha, callback) {
	var height = (0, _utils.getHeight)(buffer, width, hasAlpha);
	var workingBuffer = createImageBuffer(width, height, hasAlpha);

	(0, _utils.forEachPixel)(buffer, width, hasAlpha, function (x, y, bytePos) {
		var color = callback(x, y, bytePos, getColor(buffer, width, hasAlpha, x, y));

		var _color = _slicedToArray(color, 3);

		workingBuffer[bytePos + _constants.CHANNEL_RED] = _color[0];
		workingBuffer[bytePos + _constants.CHANNEL_GREEN] = _color[1];
		workingBuffer[bytePos + _constants.CHANNEL_BLUE] = _color[2];


		if ((0, _utils.isRGBA)(color) && hasAlpha) {
			workingBuffer[bytePos + _constants.CHANNEL_ALPHA] = color[_constants.CHANNEL_ALPHA];
		} else if (!(0, _utils.isRGBA)(color) && hasAlpha) {
			workingBuffer[bytePos + _constants.CHANNEL_ALPHA] = 0xff;
		}
	});

	return workingBuffer;
}

/**
 * Replaces the specified color on the canvas.
 */
function replaceColor(buffer, width, hasAlpha, x, y) {
	return mapPixels(buffer, width, hasAlpha, function (x, y, bytePos, color) {
		return (0, _utils.isEqualColor)(getColor(buffer, width, hasAlpha, x, y), replacee) ? replacer : color;
	});
}

/**
 * Returns with the color of the specified coordinates.
 */
function getColor(buffer, width, hasAlpha, x, y) {
	var bytePos = (0, _utils.coordinates2bytePosition)(width, hasAlpha, x, y);
	var color = [buffer[bytePos + _constants.CHANNEL_RED], buffer[bytePos + _constants.CHANNEL_GREEN], buffer[bytePos + _constants.CHANNEL_BLUE]];

	if (hasAlpha) {
		color[_constants.CHANNEL_ALPHA] = buffer[bytePos + _constants.CHANNEL_ALPHA];
	}

	return color;
}

exports.hexColorToArray = _utils.hexColorToArray;