'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.getChannelCount = getChannelCount;
exports.bytePosition2Coordinates = bytePosition2Coordinates;
exports.coordinates2bytePosition = coordinates2bytePosition;
exports.forEachPixel = forEachPixel;
exports.forEachByte = forEachByte;
exports.blendColor = blendColor;
exports.rgba = rgba;
exports.blendChannel = blendChannel;
exports.blendAlpha = blendAlpha;
exports.isRGBA = isRGBA;
exports.getAlpha = getAlpha;
exports.hasCoordinates = hasCoordinates;
exports.isEqualColor = isEqualColor;
exports.hexColorToArray = hexColorToArray;
exports.getHeight = getHeight;

var _constants = require('./constants.js');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function getChannelCount(hasAlpha) {
	return hasAlpha ? _constants.RGBA : _constants.RGB;
}

function bytePosition2Coordinates(width, hasAlpha, pos) {
	var byteWidth = width * getChannelCount(hasAlpha);
	return [Math.floor(pos % byteWidth / getChannelCount(hasAlpha)), Math.floor(pos / byteWidth)];
}

function coordinates2bytePosition(width, hasAlpha, x, y) {
	return (y * width + x) * getChannelCount(hasAlpha);
}

function forEachPixel(buffer, width, hasAlpha, callback) {
	var channelCount = getChannelCount(hasAlpha);

	for (var bytePos = 0; bytePos < buffer.length; bytePos += channelCount) {
		var _bytePosition2Coordin = bytePosition2Coordinates(width, hasAlpha, bytePos),
		    _bytePosition2Coordin2 = _slicedToArray(_bytePosition2Coordin, 2),
		    x = _bytePosition2Coordin2[0],
		    y = _bytePosition2Coordin2[1];

		callback(x, y, bytePos);
	}
}

function forEachByte(buffer, width, hasAlpha, callback) {
	for (var bytePos = 0; bytePos < buffer.length; bytePos++) {
		var _bytePosition2Coordin3 = bytePosition2Coordinates(width, hasAlpha, bytePos),
		    _bytePosition2Coordin4 = _slicedToArray(_bytePosition2Coordin3, 2),
		    x = _bytePosition2Coordin4[0],
		    y = _bytePosition2Coordin4[1];

		callback(x, y, bytePos);
	}
}

function blendColor(color1, color2) {
	var alpha1 = getAlpha(color1),
	    alpha2 = getAlpha(color2);

	var color = [blendChannel(color1[_constants.CHANNEL_RED], color2[_constants.CHANNEL_RED], alpha1, alpha2), blendChannel(color1[_constants.CHANNEL_GREEN], color2[_constants.CHANNEL_GREEN], alpha1, alpha2), blendChannel(color1[_constants.CHANNEL_BLUE], color2[_constants.CHANNEL_BLUE], alpha1, alpha2)];

	if (isRGBA(color1)) {
		color[_constants.CHANNEL_ALPHA] = blendAlpha(alpha1, alpha2);
	}

	return color;
}

function rgba(color) {
	var rgbaColor = [].concat(_toConsumableArray(color));
	rgbaColor[_constants.CHANNEL_ALPHA] = color[_constants.CHANNEL_ALPHA] ? color[_constants.CHANNEL_ALPHA] : 0xff;
	return rgbaColor;
}

function blendChannel(dstRGB, srcRGB) {
	var dstA = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0xff;
	var srcA = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0xff;

	dstRGB = dstRGB / 0xff;
	srcRGB = srcRGB / 0xff;
	var outA = blendAlpha(dstA, srcA) / 0xff;
	dstA = dstA / 0xff;
	srcA = srcA / 0xff;

	if (outA === 0) {
		return dstRGB * 0xff;
	}

	return (srcRGB * srcA + dstRGB * dstA * (1 - srcA)) / outA * 0xff;
}

function blendAlpha(dstA, srcA) {
	srcA = srcA / 0xff;
	dstA = dstA / 0xff;
	return Math.min(srcA + dstA * (1 - srcA), 1) * 0xff;
}

function isRGBA(color) {
	return color.length === 4;
}

function getAlpha(color) {
	return isRGBA(color) ? color[_constants.CHANNEL_ALPHA] : 0xff;
}

function hasCoordinates(buffer, width, hasAlpha, x, y) {
	var height = getHeight(buffer, width, hasAlpha);
	return x < width && y < height;
}

/**
 * Compares two colors.
 */
function isEqualColor(color1, color2) {
	return color1[_constants.CHANNEL_RED] === color2[_constants.CHANNEL_RED] && color1[_constants.CHANNEL_GREEN] === color2[_constants.CHANNEL_GREEN] && color1[_constants.CHANNEL_BLUE] === color2[_constants.CHANNEL_BLUE];
}

function hexColorToArray(hexColor) {
	var alpha = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
	var regex = /#([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})/,
	    _hexColor$match = hexColor.match(regex),
	    _hexColor$match2 = _slicedToArray(_hexColor$match, 4),
	    red = _hexColor$match2[1],
	    green = _hexColor$match2[2],
	    blue = _hexColor$match2[3];


	return [parseInt(red, 16), parseInt(green, 16), parseInt(blue, 16), alpha * 255];
}

function getHeight(buffer, width, hasAlpha) {
	return Math.ceil(buffer.length / width / getChannelCount(hasAlpha));
}